@@ -30,6 +30,7 @@
 """SQLAlchemy storage engine for Pygasus."""
 
 from datetime import date, datetime
+import enum
 import operator
 from pathlib import Path
 from typing import Any, Dict, List, Optional, Set, Tuple, Type, Union
@@ -60,6 +61,7 @@ from sqlalchemy.sql import select
 
 from pygasus.model import Model, Sequence
 from pygasus.storage.abc import AbstractStorageEngine
+from pygasus.storage.sql.query_builder import SQLQueryBuilder
 
 SQL_TYPES = {
     bytes: LargeBinary,
@@ -82,6 +84,7 @@ class SQLStorageEngine(AbstractStorageEngine):
 
     def __init__(self):
         super().__init__()
+        self.query_builder = SQLQueryBuilder(self)
         self.file_name = None
         self.memory = False
         self.tables = {}
@@ -179,8 +182,11 @@ class SQLStorageEngine(AbstractStorageEngine):
         # Browse model fields, creating columsn, indexes and constraints.
         indexes = []
         for name, field in model.__fields__.items():
+            model.__pygasus__[name].__model__ = model
+            model.__pygasus__[name].__storage__ = self
             o_type = field.outer_type_
             f_type = field.type_
+            info = field.field_info
             origin = get_origin(o_type)
             if origin is list or o_type is Sequence[f_type]:
                 # Analyze the back field.  If it's a list too, an
@@ -223,11 +229,34 @@ class SQLStorageEngine(AbstractStorageEngine):
                 )
                 continue
 
+            # Handle enumerations here.
+            if issubclass(f_type, enum.Enum):
+                e_type = type(list(f_type)[0].value)
+                if not all(
+                    isinstance(member.value, e_type) for member in f_type
+                ):
+                    raise ValueError(
+                        f"the enumeration {f_type} contains members 3"
+                        "of different types"
+                    )
+                invalid = info.extra.get("invalid_member", ...)
+                if invalid is ...:
+                    raise ValueError(
+                        f"field {model.__name__}.{name}: no invalid member "
+                        f"is provided for the enumeration {f_type}.  The "
+                        "invalid member is required when using enumerations "
+                        "to let Pygasus know what to do if the stored "
+                        "enumeration member doesn't match the enumeration "
+                        "class.  Provide one with "
+                        "Field(..., invalid_member=MyEnum.INVALID)"
+                    )
+                f_type = e_type
+
             sql_type = SQL_TYPES.get(f_type)
             if sql_type is None:
                 raise ValueError(f"unknown type: {name} ({f_type})")
             else:
-                extra = field.field_info.extra
+                extra = info.extra
                 primary_key = extra.get("primary_key", False)
                 unique = extra.get("unique", False)
                 index = extra.get("index", False)
@@ -320,6 +349,8 @@ class SQLStorageEngine(AbstractStorageEngine):
 
                 exclude.add(name)
                 continue
+            elif issubclass(f_type, enum.Enum):
+                sql[name] = attrs[name].value
 
             pk = info.extra.get("primary_key", False)
             if pk and issubclass(f_type, int):
@@ -425,6 +456,35 @@ class SQLStorageEngine(AbstractStorageEngine):
                 additional[col_name] = index
         return self.insert(model, attrs, additional)
 
+    def select(self, model: Type[Model], query):
+        """Select one or more model instances with the specified query.
+
+        Args:
+            model (subclass of Model): the model object.
+            query: the query object by which to filter.
+
+        Returns:
+            instances (list of Model): the model isntances or None.
+
+        """
+        model_name = getattr(
+            model.__config__, "model_name", model.__name__.lower()
+        )
+        table = self.tables[model_name]
+        models = [model]
+        columns, tables = self._get_columns_for(model)
+        sql = select(*columns)
+        for join in tables:
+            sql = sql.join(table)
+        sql = sql.where(query)
+
+        # Send the query.
+        rows = self.connection.execute(sql).fetchall()
+        return [
+            self._build_objects_from_row(dict(row), models, first=True)
+            for row in rows
+        ]
+
     def get(self, model: Type[Model], **kwargs):
         """Get a model instance with the specified arguments.
 
@@ -565,6 +625,10 @@ class SQLStorageEngine(AbstractStorageEngine):
                 value = getattr(instance, name)
                 sql_primary_keys.append(getattr(sql_table.c, name) == value)
 
+        # Handles enum field.
+        if isinstance(new_value, enum.Enum):
+            new_value = new_value.value
+
         # Send the query.
         sql_columns = {key: new_value}
         update = (
@@ -681,8 +745,8 @@ class SQLStorageEngine(AbstractStorageEngine):
                 o_type = field.outer_type_
                 f_type = field.type_
                 pk = info.extra.get("primary_key", False)
+                value = row.get(f"{model_name}_{name}", ...)
                 if pk:
-                    value = row.get(f"{model_name}_{name}", ...)
                     if value is not ...:
                         pks.append(value)
                     attrs[name] = row[f"{model_name}_{name}"]
@@ -699,6 +763,16 @@ class SQLStorageEngine(AbstractStorageEngine):
                     )
                     others.append(obj)
                     attrs[name] = obj
+                elif issubclass(f_type, enum.Enum):
+                    try:
+                        value = f_type(value)
+                    except ValueError:
+                        invalid_key = getattr(
+                            model.__config__, "invalid_enum_key", "INVALID"
+                        )
+                        value = getattr(f_type, invalid_key)
+                    finally:
+                        attrs[name] = value
                 elif o_type is not Sequence[f_type]:
                     attrs[name] = row[f"{model_name}_{name}"]
 
@@ -726,3 +800,15 @@ class SQLStorageEngine(AbstractStorageEngine):
             objs = [None]
 
         return objs[0] if first else objs
+
+    def equal(self, field, other):
+        """Compare field to other."""
+        model = field.__model__
+        model_name = getattr(
+            model.__config__, "model_name", model.__name__.lower()
+        )
+        table = self.tables[model_name]
+        return getattr(table.c, field.name) == other
+
+    def eq(self, field, other):
+        """Compare field to other."""
